CON

  _clkmode = xtal1+pll16x
  _xinfreq = 5_000_000

  CMD_PLAY            = %00000001
  CMD_STOP            = %00000010
  CMD_PAUSE           = %00000011
  CMD_VOLUME          = %00000100
  CMD_BASSANDTREBLE   = %00000101
  PLAYING             = %00000110
  CMD_NEXTFILE        = %00000111
  CMD_PREVIOUSFILE    = %00001000
  IDLE                = %00001001
  EOF                 = %00001010
  ERROR01             = %00001011       'requested file does not exist
  ERROR02             = %00001100       'error while cancelling playback  (software reset recommended)

  '' Version 0.5 contains PIN changes 
  
OBJ

  text    : "parallel_lcd_pexp_mod"
  str     : "util_strings"
  sdfat   : "fsrw"
  remote  : "ir_reader_nec"
  decoder : "vs1053 mp3"
  time    : "timing"

VAR
byte status, musicbuffer[32]
byte cursorpos, cogNO
long lastplayed, currentsource, help
'long maxT
long playstack[32]
long setting[10]
byte volume, hasmeta
word seconds, playpos
long namebuffer, filesize, Poss, fileposition
byte filelist[3315]

byte strMeta[128]

byte strTitle[31]
byte strArtist[31]
byte strAlbum[31]

PUB start | keycode, temp
                         
dira[6]~
dira[7]~~
dira[30]~~                    'These lines are needed for the speaker "anti-pop" mechanism
outa[30]:=1                   'The output is pulled down to ground while
outa[7]:=1                    'the output capacitors of the line out connection
time.pause1ms(100)            'are charging
outa[30]:=0                   'When they are fully charged, allow signal output

help:=-1                      'initializing some variables...
currentsource:=-1
lastplayed:=-1
filelist:=0
status:=0


setting[0]:=0                'bassboost setting
setting[1]:=0                'treble setting
setting[2]:=0                'balance setting (0 means left and right channel at the same level)
volume:=235                  'standard volume (as I'm using the output as line out, this is pretty high)


text.start                   'starting the LCD...
remote.init(15, 0, 2, true)  '...the IR decoder...
decoder.init(3,4,2,1,5)      '...and the VS1053 decoder chip
decoder.WriteReg(3,38912)    'setting the clock divider of the VS1053


remote.fifo_flush            'clear the IR fifo buffer
decoder.setvolume(volume,setting[2])   'set the standard volume defined above
text.str(string("Musikplayer bereit")) 'printing a message to see when the player is ready for first input

repeat
    keycode := remote.fifo_get   'try a get from fifo
    if keycode == -1             'empty try again
      case currentsource                  
        -1:
          next
        0:               'If we played a file from SD card before, we don't need to mount it again
          fileposition:=menu1(@title1, @filelist, 13, 255, fileposition / 13) * 13   
          remote.fifo_flush
          status:=CMD_PLAY           'set the status to play...
          Poss:=0
          cogNO:=cognew(play(@status),@playstack) '...and start a new cog
          playdisplay                'call playdisplay to display information during playback
        1:
          Freigabeliste              'ToDo: Look for shared files in the LAN
        2:
          Streamliste                'ToDo: Display a list of internet radio streams (I'll need more RAM first!)
        3:
          settings
      next
    elseif keycode == $0A
      settings
    elseif keycode == $0C            '"source select" button is pressed
        case currentsource:=source
          0:                         'chose SD card as source
           temp:=checkforSD
           if temp==0
            fileposition:=menu1(@title1, @filelist, 13, 255, 0) * 13 'if an SD card was found, show a list of files
            remote.fifo_flush
            status:=CMD_PLAY
            Poss:=0
            cogNO:=cognew(play(@status),@playstack)
            playdisplay
           else
            message(0,temp)               'display an error message, if no SD card was found
            next
          1:                         'ToDo: chose shared files
           Freigabeliste
          2:                         'ToDo: chose internet radio streams
           Streamliste
    elseif keycode == $5F or keycode==$5E
      setvolume
    
      'text.out($00)
      'text.dec(keycode)         'show code      
    'text.out(" ") 
                                'device code is in low 16 bits
    'text.hex(remote.fifo_get_lastvalid,8)

PRI source      'This procedure is called to select a data input source

  return menu1(@title2, @qname, 17, 3, 0)  

PRI checkforSD | NOofFiles, temp   'This procedure tries to mount the SD card and will return an errorcode if it fails

  NOofFiles:=0
  temp:=\sdfat.mount_explicit(12,11,10,9)
  if temp==0
    'waitcnt(clkfreq+cnt)
    sdfat.opendir
    repeat until sdfat.nextfile(@filelist[NOofFiles*13])==-1 OR NOofFiles>255
     if str.indexOf(@filelist[NOofFiles*13], string(".MP3")) <> -1 OR str.indexOf(@filelist[NOofFiles*13], string(".M4A")) <> -1
      NOofFiles++
     elseif str.indexOf(@filelist[NOofFiles*13], string(".MID")) <> -1 OR str.indexOf(@filelist[NOofFiles*13], string(".WAV")) <> -1
      NOofFiles++
     elseif str.indexOf(@filelist[NOofFiles*13], string(".WMA")) <> -1 OR str.indexOf(@filelist[NOofFiles*13], string(".OGG")) <> -1
      NOofFiles++
    return 0
  else
    return temp   

PRI playlist

  'ToDo

PRI settings | keycode, mode     'This is the settings menu where bass, treble and balance can be changed

  cursorpos:=menu1(@title3, @sname, 8, 3, 0)

  case cursorpos                 '...jump to a corresponding submenu

    0:
      text.str(string("B",%11100001,"sse:"))
      text.out(" ")
      text.dec(setting[cursorpos])
      mode:=0
      text.statusbar(setting[cursorpos],3,mode)

    1:
      text.str(string("H", %11101111, "hen:"))
      text.out(" ")
      text.dec(setting[cursorpos])
      mode:=0
      text.statusbar(setting[cursorpos],3,mode)

    2:
      text.str(string("Balance:"))
      text.out(" ")
      text.dec(setting[cursorpos])
      mode:=1
      text.statusbar(setting[cursorpos],3,mode)

  remote.fifo_flush  
      
  repeat
    keycode := remote.fifo_get
    if keycode == -1
      next
    elseif (keycode == $5E AND mode==0 AND setting[cursorpos]>0)
      setting[cursorpos]--
      text.statusbar(setting[cursorpos],3,mode)
      if status == PLAYING
        status:=CMD_BASSANDTREBLE
      elseif status == CMD_STOP OR status == CMD_PAUSE 
        decoder.setbassboost(setting[0])
        decoder.settreble(setting[1])
    elseif (keycode == $5E AND mode==1 AND setting[cursorpos]>-20)
      setting[cursorpos]--
      text.statusbar(setting[cursorpos],3,mode)
      if status == PLAYING
        status:=CMD_VOLUME
      elseif status == CMD_STOP OR status == CMD_PAUSE
        decoder.setvolume(volume,setting[2])
    elseif (keycode == $5F AND mode==0 AND setting[cursorpos]<15)
      setting[cursorpos]++
      text.statusbar(setting[cursorpos],3,mode)
      if status == PLAYING
        status:=CMD_BASSANDTREBLE
      elseif status == CMD_STOP OR status == CMD_PAUSE
        decoder.setbassboost(setting[0])
        decoder.settreble(setting[1])
    elseif (keycode == $5F AND mode==1 AND setting[cursorpos]<20)
      setting[cursorpos]++
      text.statusbar(setting[cursorpos],3,mode)
      if status == PLAYING
        status:=CMD_VOLUME
      elseif status == CMD_STOP OR status == CMD_PAUSE  
        decoder.setvolume(volume,setting[2])
    elseif keycode == $5D
      text.out($00)
      quit
    
      
    if cursorpos==0
        text.pos(6,1)
    elseif cursorpos==1
        text.pos(6,1)
    elseif cursorpos==2
        text.pos(8,1)
    else
      next
  
    text.out(" ")
    text.dec(setting[cursorpos])
    text.str(string("  "))

  return

pri setvolume | keycode, temp         'This procedure is called if the user wants to change the volume

    remote.fifo_flush
    temp:=0
    
    text.out($00)
    text.str(string("Lautst", %11100001,"rke:"))
    text.statusbar(volume,3,2)        'some really cool graphics :-P
    text.pos(11,1)
    text.out(" ")
    text.dec(volume)                  'display current volume level as a number
    text.str(string("  "))

    repeat
      keycode := remote.fifo_get
      if keycode==-1
        temp++
        waitcnt(clkfreq+cnt)   'This menu closes itself after some time if no button is pressed anymore
        if temp==5
          text.out($00)
          return
        next
      elseif keycode==$5F AND volume<255  'if "volume up" button is pushed and volume is below maximum...
        temp:=0
        volume+=5                          '...allow an increase of the volume by 5
        text.statusbar(volume,3,2)        'again graphics
        if status == PLAYING
          status:=CMD_VOLUME              'tell the cog which is sending the music data to change volume 
        elseif status == CMD_STOP OR status == CMD_PAUSE  'if no music is playing...
          decoder.setvolume(volume,setting[2])            '...volume change can be made here
      elseif keycode==$5E AND volume>0    'if "volume down" button is pushed and volume is above minimum...
        temp:=0
        volume-=5                          '...allow a decrease of the volume by 5
        text.statusbar(volume,3,2)
        if status == PLAYING
          status:=CMD_VOLUME              'tell the cog which is sending the music data to change volume
        elseif status == CMD_STOP OR status == CMD_PAUSE  'if no music is playing...
          decoder.setvolume(volume,setting[2])            '...volume change can be made here
     text.pos(11,1)
     text.out(" ")
     text.dec(volume)
     text.str(string("  "))               
  
pri Freigabeliste

  return

pri Streamliste

  return

pri message(index, extrainfo) | keycode           'Used for error messages

remote.fifo_flush
text.out($00)

if index==0                            'print this if no SD card was found
  {text.pos(0,1)
  text.str(string("Keine SD-Karte!"))
  text.pos(0,3)
  text.str(string("Bitte andere Quelle"))
  text.pos(5,4)
  text.str(string("w",%11100001,"hlen"))}
  text.pos(0,1)
  if extrainfo == -20
    text.str(string("no FAT volume"))
  elseif extrainfo == -21
    text.str(string("bad bytes/sector"))
  elseif extrainfo == -23
    text.str(string("not 2 FAT"))
  elseif extrainfo == -24
    text.str(string("bad signature"))
  else
    text.dec(extrainfo)
  repeat until keycode == $5D or keycode == $10          'and wait until return/play is pressed
    keycode:=remote.fifo_get
elseif index==1
  text.str(string("ERROR02"))
  text.pos(0,3)
  text.str(string("Neustart notwendig!"))
  repeat
elseif index==2
  text.str(string("ERROR03"))
  text.pos(0,3)
  text.str(string("ung",%11110101,"ltiges Format"))
  waitcnt(2*clkfreq+cnt)
return

PRI _readTags | i, j, marker, dummy[8], size[4], metadata[256]

  playpos:=0
  metadata[255]:=0
  'sdfat.popen(filename, "r")
  if \sdfat.seek(sdfat.get_filesize - 128) < 0                ' seek to the start of a possible ID3v1 TAG
    return false
  
  if sdfat.pgetc == "T" and sdfat.pgetc == "A" and sdfat.pgetc == "G"
    ' extract tags
    sdfat.pread(@strTitle, 30)                             ' read title
    sdfat.pread(@strArtist, 30)                            ' read artist
    sdfat.pread(@strAlbum, 30)                             ' read album

    strTitle[30] := 0
    strArtist[30] := 0
    strAlbum[30] := 0

    ' generate Artist - Title metadata string
      
    result := true
    
  else                                                          ' no ID3v1 tags ; check for ID3v2.3
    \sdfat.seek(0)
    if sdfat.pgetc == "I" and sdfat.pgetc == "D" and sdfat.pgetc == "3" and sdfat.pgetc == $03   
      sdfat.pread(@dummy,2)                       ' don't care about revision and flags
      sdfat.pread(@size,4)                         
      if size >= 256
        return false                           ' tag size exceeds 256 bytes
      else
        Metadata:=0
        sdfat.pread(@Metadata, size)
        size:=0
        i:=str.indexOf(@Metadata, string("TALB"))
        if i<>-1
          bytemove(@size, @Metadata[i+4], 4)
          if size =< 30
           bytemove(@strAlbum, @Metadata[i+8], size)
           strAlbum[size] := 0
        i:=str.indexOf(@Metadata, string("TIT2"))
        if i<>-1
          bytemove(@size, @Metadata[i+4], 4)
          if size =< 30
           bytemove(@strTitle, @Metadata[i+8], size)
           strTitle[size] := 0
        i:=str.indexOf(@Metadata, string("TPE1"))
        if i<>-1
          bytemove(@size, @Metadata[i+4], 4)
          if size =< 30
           bytemove(@strArtist, @Metadata[i+8], size)
           strArtist[size] := 0
        result := true
    else
     return false   
     'TALB frame : album title
     'TIT2 frame : song title
     'TPE1 frame : lead performer(s)
     'TLEN frame : length in milliseconds
     'TYER frame : year
    {else                                                          'last chance: Itunes Tag
      marker:=%0000
      Metadata:=0
      \sdfat.seek(4)
      if sdfat.pgetc == "f" and sdfat.pgetc == "t" and sdfat.pgetc == "y" and sdfat.pgetc == "p"
        sdfat.pread(@Metadata, 256)
        'repeat 
          if marker & %0001 == 0
            i:=str.indexOf(@Metadata, string(%10101001, "alb"))
            if i<>-1
               bytemove(@size, @Metadata[i+4], 4)
                if size =< 30
                  bytemove(@strAlbum, @Metadata[i+20], size)
                  strAlbum[size] := 0
                  marker += %0001
          if marker & %0010 == 0
            i:=str.indexOf(@Metadata, string(%10101001, "art"))
            if i<>-1
              bytemove(@size, @Metadata[i+4], 4)
              if size =< 30
                bytemove(@strArtist, @Metadata[i+20], size)
                strArtist[size] := 0
                marker += %0010
          if marker & %0100 == 0
            i:=str.indexOf(@Metadata, string(%10101001, "nam"))
            if i<>-1
              bytemove(@size, @Metadata[i+4], 4)
              if size =< 30
                bytemove(@strTitle, @Metadata[i+20], size)
                strTitle[size] := 0
                marker += %0100
          {j:=str.indexOf(@Metadata, string("mdat"))
          if j<>-1
            'playpos+=i
            quit}
         'Metadata <<= 1024
         'bytemove(@Metadata, @Metadata[128], 128)
         'sdfat.pread(@Metadata[128], 128)
         'playpos+=128 
        result := true
      else
        return false }            

  i := strsize(@strArtist)
  bytemove(@strMeta, @strArtist, i)
  bytemove(@strMeta[i], string(" - "), 3)
  i += 3
  bytemove(@strMeta[i], @strTitle, strsize(@strTitle) + 1)                                   

pri playdisplay | keycode, i, t, scrollpos', length, musicstartbyte, currlength     'This procedure keeps updating the screen during playback
                                       'and waits for/responds to IR commands
  remote.fifo_flush
  text.out($00)
  'text.str(@filelist[fileposition])    'Display the filename
  text.pos(0,3)
  seconds:=0
  repeat
    text.pos(0,1)
        if hasmeta
            ' scrolly metadata
            if strsize(@strMeta) > 20
              i := scrollpos++    ' scroll the display
            else
              i := 0              ' don't scroll (song title fits on the line)          
            repeat 20
              if i < strsize(@strMeta)
                text.out(strMeta[i++])
              else
                text.out(" ")
            if scrollpos => strsize(@strMeta)
              scrollpos := 0
        else
          text.str(@filelist[fileposition])  'ToDo: Find a way to retrieve the filename once after returning from another menu
    time.pause1ms(20) 
    keycode:=remote.fifo_get
    'text.pos(0,2)
    'text.out(%00010000)     'This isn't working! Should output a "play" symbol. (Maybe not in CGROM)  
    text.pos(2,2)
    if (seconds/60)<10       'Leave a free space, if minutes are less than 10
      text.out(" ")          'Should prevent a jump of the time display (but I've got no song longer than 10 minutes)
    text.dec(seconds/60)     'Display minutes of decoded data
    text.out(":")
    if (seconds//60)<10      'If the seconds are less than 10
      text.out("0")          'then print a zero first
    text.dec(seconds//60)    'Display the seconds
    {text.out("/")
    text.dec(length/60)
    text.out(":")
    if (length//60)<10      'If the seconds are less than 10
      text.out("0")          'then print a zero first
    text.dec(length//60)    'Display the seconds}
    if keycode == $1B        'Check if there is input from the IR remote control
      status:=CMD_PAUSE
    elseif keycode == $10
      status:=CMD_PLAY
    elseif keycode == $5F or keycode == $5E
      setvolume
    elseif keycode == $0A
      settings
    elseif keycode == $18 AND status == PLAYING
      decoder.setmode(8,1)
      status:=CMD_NEXTFILE
    elseif keycode == $19 AND status == PLAYING
      decoder.setmode(8,1)
      status:=CMD_PREVIOUSFILE
    elseif keycode == $1A
      decoder.setmode(8,1)      'set the SM_CANCEL bit to tell the decoder chip that we want to stop playback
      status:=CMD_STOP          'and also tell the cog which is sending the music data
      filesize:=0
      quit
    if status == ERROR01 OR status == ERROR02 OR status == EOF   
      'text.out($00)
      'text.str(@filelist[fileposition])
      'repeat
      quit  
    {repeat until seconds > 1                      'trying to calculate total song length; not very accurate
      musicstartbyte:=Poss
      'currlength:=0
      'length:=0
      t:=0
    currlength:=currlength+((seconds-2)*(filesize-musicstartbyte)/(Poss-musicstartbyte))
    if currlength > 0
      length:=currlength/t++}
                  

  repeat while status == CMD_STOP
  if status == ERROR02
    message(1,0)
  if seconds == 0 AND status == EOF
    message(2,0)

  return    

pri play(state) | h, Bytes, Read, loop               

status:=PLAYING
'lastplayed:=0
if currentsource == 0 AND sdfat.popen(@filelist[fileposition],"r")<>-1
  filesize:=sdfat.get_filesize
  hasmeta:=_readTags
  sdfat.seek(0)
  'if playpos<>0
    'sdfat.seek(playpos)
  decoder.WriteReg(4,0)                                 'Clear the DECODE_TIME register when a new file is opened
  repeat
    if ina[6] == 1
      Bytes := sdfat.pread(@musicbuffer,32)
      Poss += Bytes
      if Bytes < 32
        repeat Bytes
          decoder.WriteDataByte(musicbuffer[Read++])
        quit
      if status <> PLAYING   'If you check for every possible status in the main loop, bandwidth will dramatically decrease
        if status == CMD_STOP AND decoder.ReadModeBit(8)==0  'Check if SM_CANCEL bit has cleared
          quit
        elseif status == CMD_NEXTFILE AND decoder.ReadModeBit(8)==0
          cleanstop
          fileposition+=13                                   'Look for the next file in the filelist
          if sdfat.popen(@filelist[fileposition],"r")<>-1    'and try to open it
            filesize:=sdfat.get_filesize
            decoder.WriteReg(4,0)                            'Clear the DECODE_TIME register
            status:=PLAYING                                  '...set the new status...
            next                                             '...and return to the main loop
          else
            status:=ERROR01                                  'if the file doesn't exist
            quit                                             'exit the main loop and stop the cog
        elseif status == CMD_PREVIOUSFILE AND decoder.ReadModeBit(8)==0  'same for previous file
          cleanstop
          fileposition-=13
          if sdfat.popen(@filelist[fileposition],"r")<>-1
            filesize:=sdfat.get_filesize
            decoder.WriteReg(4,0)
            status:=PLAYING
            next
          else
            status:=ERROR01
            quit
        elseif status == CMD_PAUSE       'I implemented "pause" by creating a buffer underrun...
          repeat until status == CMD_PLAY   '...Not a very nice way, but it's working
          status:=PLAYING                
        elseif status == CMD_VOLUME      'If the volume is changed during playback...
          decoder.setvolume(volume,setting[2])  '...this cog sets the new value
          status:=PLAYING
        elseif status == CMD_BASSANDTREBLE  'same for bass and treble
          decoder.setbassboost(setting[0])
          decoder.settreble(setting[1])
          status:=PLAYING        
      decoder.WriteDataBuffer(@musicbuffer)  'send the bytes read from the SD card to the decoder chip
      if (Poss//1024)                        'update the decode time only after every 1024 bytes...
        seconds:=decoder.DecodeTime          '...for a 128kBit/s mp3 that's every sixteenth of a second

cleanstop
cogstop(cogNO)

pri cleanstop | endFillByte, loop1, loop2

    endFillByte := decoder.VSReadRAM($1e06)            'Read the endFillByte value
    repeat loop1 from 0 to 2051
      repeat until ina[6] == 1
      decoder.WriteDataByte(endFillByte)

    if status == PLAYING         
      decoder.setmode(8,1)      'set the SM_CANCEL bit
      repeat loop1 from 0 to 63
        repeat loop2 from 0 to 31
          repeat until ina[6] == 1
          decoder.WriteDataByte(endFillByte)
        if decoder.ReadModeBit(8)==0
          quit
    
    if decoder.ReadReg(8) <> 0 OR decoder.ReadReg(9) <> 0
      status:=ERROR02
      return
    elseif status <> PLAYING
        status:=IDLE
        return
    elseif status == PLAYING
        status:=EOF
        return

PRI menu1(menutitle, data, entrylength, NOofentries, position) | x, y, loop, keycode

  remote.fifo_flush   
  text.out($00)
  cursorpos:=2
  y:=position

  if position > NOofentries OR entrylength > 18      'entry length should not exceed 18 characters + 2 for the cursor
    text.str(string("Error!"))
    repeat

  if byte[data][(position+1)*entrylength] == 0       'This will ensure that no empty spaces are displayed 
    y:=position - 2                                  'if the data array contains less entries than the NOofentries
    cursorpos:=4                                     'for example: returning to the filelist after playing the last file of the list 
  elseif byte[data][(position+2)*entrylength  ] == 0
    y:=position - 1
    cursorpos:=3  
  else
    y:=position
  
  text.str(menutitle)                            
  text.pos(0,cursorpos)                          'set the cursor position to the specified line
  text.str(string(">>"))                         
  repeat x from 0 to 2                               
    text.pos(2,x+2)                              'x+2 represents the LCD line number 
    text.str(@byte[data][(y+x)*entrylength])            'print string at position y (+x for the next element) of the data array 
    repeat loop from entrylength to 18           'clear rest of the line
      text.out(" ")

  repeat
    keycode := remote.fifo_get         'Check if there is input from the IR remote control
    if keycode == -1
      next

    elseif keycode == $0A    
      return settings

    elseif keycode == $0E              
      if cursorpos<4                   'let the cursor go down the list... 
        text.pos(0,cursorpos)          '..while it hasn't reached the last line of the display 
        text.str(string("  "))
        cursorpos++
        text.pos(0,cursorpos)
        text.str(string(">>"))
      elseif byte[data][(y+3)*entrylength]<>0 AND y+3 <> NOofentries    'otherwise move all entries up by one...
        y+=1
        repeat x from 0 to 2
          text.pos(2,x+2)
          text.str(@byte[data][(y+x)*entrylength])         '...and add a new entry at the bottom
          if entrylength <> 13 AND entrylength <> 18
            repeat loop from entrylength to 18
              text.out(" ")
          elseif entrylength == 13
            text.str(string("     "))
 
    elseif keycode == $0F  
      if cursorpos>2                   'let the cursor go up the list...
        text.pos(0,cursorpos)          '..while it hasn't reached the second line of the display
        text.str(string("  "))
        cursorpos--
        text.pos(0,cursorpos)
        text.str(string(">>"))
      elseif y>0                       'otherwise move all entries down by one...
        y-=1                 '...so that the entry at the bottom vanishes...
        repeat x from 0 to 2
          text.pos(2,x+2)
          text.str(@byte[data][(y+x)*entrylength])         '... and a new one is shown at the top
          if entrylength <> 13 AND entrylength <> 18
            repeat loop from entrylength to 18
              text.out(" ")
          elseif entrylength == 13
            text.str(string("     "))
                
    elseif keycode == $5D or keycode == $10              'an entry is selected...
      return y+(cursorpos-2)      '...so return the list position of the entry (without regarding the length of this entry)
    elseif keycode == $5F or keycode == $5E
      setvolume
    elseif keycode == $0C
      return source

DAT

title1 byte "Dateiliste:",0
title2 byte "Bitte Quelle w",%11100001,"hlen:",0
title3 byte "Einstellungen:",0

qname
  byte "SD-Karte        ",0
  byte "LAN Freigaben   ",0
  byte "Internet Streams",0

sname
  byte "B", %11100001, "sse  ",0
  byte "H", %11101111, "hen  ",0
  byte "Balance",0